# -*- coding: utf-8 -*-
"""Module containing the class for the client
"""
# import basic stuff
import time
from typing import List, Optional

# import own stuff
import tlsmate.client_auth as client_auth
import tlsmate.client_state as client_state
import tlsmate.config as conf
import tlsmate.connection as conn
import tlsmate.msg as msg
import tlsmate.recorder as rec
import tlsmate.structs as structs
import tlsmate.tls as tls

# import other stuff


class Client(object):
    """The class representing a TLS client

    Connections are initiated by the TLS client, and it also stores data across
    several connections, like session tickets.

    A TLS client has a dedicated client profile
    (:obj:`tlsmate.client_state.ClientProfile`) which controls the TLS
    connections, e.g. the supported TLS versions, the supported cipher suites
    and other attributes are defined there (e.g. set of supported groups,
    whether encrypt-then-mac is supported, etc.)

    The client profile is used for setting up TLS messages in case only the
    message class and not a message instance is provided in the test case.
    """

    # The following properties are deprecated and will likely be removed with the next
    # major release. They are kept here for backward compatibility reasons.
    # TODO: remove properties with the next major release.
    @property
    def alert_on_invalid_cert(self) -> bool:
        """Controls the behavior in case a certificate or
        the complete certificate chain cannot successfully be validated. If
        True, the connection will be closed with a fatal alert. If False, the
        connection continues. The latter is useful for scanning a server, as
        the handshake would be aborted prematurely otherwise.
        """
        return self._alert_on_invalid_cert

    @alert_on_invalid_cert.setter
    def alert_on_invalid_cert(self, val: bool) -> None:
        self._alert_on_invalid_cert = val
        self._config.set("alert_on_invalid_cert", val)

    @property
    def server_issues(self) -> List[structs.Malfunction]:
        return self.session.server_issues

    def __init__(
        self,
        config: conf.Configuration,
        recorder: rec.Recorder,
        client_auth: client_auth.ClientAuth,
    ) -> None:
        """Initialize the client object

        Args:
            tlsmate: the tlsmate application object.
        """
        self._config = config
        self._recorder = recorder
        self._client_auth = client_auth

        self._host: str = self._config.get("host")
        self._port: int = self._config.get("port")
        self._sni: str = self._config.get("sni") or self._host
        self.profile: client_state.ClientProfile = client_state.create_profile(
            tls.Profile.MODERN
        )
        self.session = client_state.SessionState(
            self._host, self._port, self._sni, self._recorder
        )
        self.alert_on_invalid_cert = True

    def init_profile(
        self, profile_values: Optional[structs.ProfileValues] = None
    ) -> None:
        """Resets the client profile to a very basic state

        :note:
            At least the versions and the cipher_suite list must be provided before
            this profile can be used.

        Compression methods is set to [:obj:`tlsmate.tls.CompressionMethod.NULL`], and
        by default the sni extension is enabled. Everything else is empty or disabled.

        Arguments:
            profile_values: the profile values to additionally use to
                initialize the client profile
        """

        self.profile = client_state.init_profile(profile_values)

    def set_profile(self, profile: tls.Profile) -> None:
        """Initializes the client according to the given profile.

        The following profiles are supported:

        - :obj:`tlsmate.tls.Profile.INTEROPERABILITY`

          - TLS Versions 1.0 - 1.3
          - ECDHE cipher & RSA-based key transport
          - AESGCM, AES, CHACHA_POLY and 3DES as last resort
          - Signature algorithms: ECDSA+SHA1, RSA PKCS1+SHA1 as last resort
          - Resumption, encrypt-then-mac, extended-master-secret
          - pskmode psk_dhe

        - :obj:`tlsmate.tls.Profile.LEGACY`

          - TLS Versions 1.0 - 1.2
          - ECDHE cipher & DHE & RSA-based key transport
          - AESGCM, AES, CHACHA_POLY and 3DES as last resort
          - Signature algorithms: ECDSA+SHA1, RSA PKCS1+SHA1 as last resort
          - Resumption, encrypt-then-mac, extended-master-secret

        - :obj:`tlsmate.tls.Profile.MODERN`

          - TLS Versions 1.2 + 1.3
          - ECDHE cipher
          - AESGCM, CHACHA_POLY
          - signatures: ECDSA + PSS_RSAE
          - Resumption, encrypt-then-mac, extended-master-secret
          - pskmode psk_dhe

        - :obj:`tlsmate.tls.Profile.TLS13`

          - TLS Version 1.3
          - AESGCM + CHACHA_POLY
          - pskmode psk_dhe

        Arguments:
            profile (:obj:`tlsmate.tls.Profile`): the profile to which the client
                shall be initialized.
        """

        self.profile = client_state.create_profile(profile)

    def create_connection(
        self,
        host: Optional[str] = None,
        port: Optional[int] = None,
        sni: Optional[str] = None,
    ) -> conn.TlsConnection:
        """Create a new connection object

        Arguments:
            host: the host to contact. If not given, the host is taken from the
                configuration. The given string can be a URL or an IP address.
                port (int): the port number

        Returns:
            the created connection object
        """

        target_host = host or self._host
        target_sni = sni or host or self._sni or self._host
        self._port = port or self._port

        if target_host != self._host or target_sni != self._sni:
            self._host = target_host
            self._sni = target_sni
            self.session = client_state.SessionState(
                target_host, self._port, self._sni, self._recorder,
            )

        self.session.port = self._port

        interval = self._config.get("interval")
        if interval:
            time.sleep(interval / 1000)

        return conn.TlsConnection(
            profile=self.profile,
            session=self.session,
            config=self._config,
            recorder=self._recorder,
            client_auth=self._client_auth,
        )

    def get_sni(self) -> str:
        """Returns the sni.

        This method is deprecated and will likely be removed with the next
        major release. It is kept here for backward compatibility reasons.

        Returns:
            the sni
        """

        # TODO: remove method with the next major release
        return self._sni

    def client_hello(self) -> msg.ClientHello:
        """Constructs a client hello dependent on the client profile and the session.

        This method is deprecated and will likely be removed with the next
        major release. It is kept here for backward compatibility reasons.

        Returns:
            the instantiated ClientHello message
        """

        # TODO: remove method with the next major release
        return msg.client_hello(
            self.profile, self.session, self._recorder, self._client_auth
        )



# -*- coding: utf-8 -*-
"""Module containing the class for the configuration
"""
# import basic stuff
import os
import logging
from pathlib import Path
from typing import Dict, Any, Optional, ItemsView, Union

# import own stuff
import tlsmate.structs as structs

# import other stuff
import configparser

config_logging = structs.ConfigItem("logging", type=str, default="error")
config_host = structs.ConfigItem("host", type=str, default="localhost")
config_port = structs.ConfigItem("port", type=int, default=443)
config_interval = structs.ConfigItem("interval", type=int, default=0)
config_key_log_file = structs.ConfigItem("key_log_file", type=str, default=None)
config_progress = structs.ConfigItem("progress", type=bool, default=False)
config_sni = structs.ConfigItem("sni", type=str, default=None)
config_ca_certs = structs.ConfigItem("ca_certs", type="file_list")
config_client_key = structs.ConfigItem("client_key", type="file_list")
config_client_chain = structs.ConfigItem("client_chain", type="file_list")
config_crl = structs.ConfigItem("crl", type=bool, default=True)
config_ocsp = structs.ConfigItem("ocsp", type=bool, default=True)
config_plugin = structs.ConfigItem("plugin", type="str_list", default=None)
config_recorder_delay = structs.ConfigItem("recorder_delay", type=bool, default=True)
config_proxy = structs.ConfigItem("proxy", type=str, default=None)
config_ipv6_preference = structs.ConfigItem("ipv6_preference", type=bool, default=False)


class Configuration(object):
    """Class representing the configuration for tlsmate.

    The configuration is taken from the following sources (the list is ordered
    according to the priority, first item has the least priority):

    * The hard coded default values
    * From the ini-file, if present. If the ini-file is not specified via the
      CLI option, the file .tlsmate.ini in the user's home directory will be used,
      if present.
    * Environment variables. They need to be given in upper cases and must start with
      `TLSMATE_` followed by the name of the setting in upper cases.
    * From the command line interface parameters

    Example:
        Specifying the logging option:

        Via command line::

            tlsmate --logging=debug ...

        Via Enviroment variable::

            export TLSMATE_LOGGING=debug

        Via ini-file::

            [tlsmate]
            logging = debug
    """

    def __init__(self) -> None:
        self._config_section: Optional[configparser.SectionProxy] = None
        self._config_section_read = False
        self._config: Dict[str, Any] = {}
        self._descr: Dict[str, structs.ConfigItem] = {}

        # register configurations which are essential in the core
        # part of tlsmate.
        for item in [
            config_logging,
            config_host,
            config_port,
            config_interval,
            config_key_log_file,
            config_progress,
            config_sni,
            config_ca_certs,
            config_client_key,
            config_client_chain,
            config_crl,
            config_ocsp,
            config_plugin,
            config_recorder_delay,
            config_proxy,
        ]:
            self.register(item)

    def _str_to_filelist(self, string):
        """Resolves a string of files paths.

        Multiple file paths are separated by a colon. If the path is a relative
        path, it is expanded to an absolute path, taking the directory of the
        config file as a base (which is available in self._config_dir).

        Arguments:
            string (str): the list of file paths as a string, multiple paths are
                separated by a colon.

        Returns:
            list of str: the list of resolved absolute paths.
        """

        ret = []
        for val in string.split(","):
            val = val.strip()
            if not val.startswith("/"):
                val = str(self._config_dir / val)
            ret.append(val)
        return ret

    _format_option = {
        bool: lambda self, x: x.lower() not in ["0", "off", "no", "false"],
        int: lambda self, x: int(x),
        "file_list": _str_to_filelist,
        "str_list": lambda self, x: [token.strip() for token in x.split(",")],
    }

    def _init_config_section(self, ini_file: Optional[Union[str, Path]]) -> None:
        if self._config_section_read:
            return

        self._config_section_read = True
        if ini_file is None:
            file_path = Path.home() / ".tlsmate.ini"
            if not file_path.is_file():
                return

        else:
            file_path = Path(ini_file)
            if not file_path.is_absolute():
                file_path = Path.cwd() / ini_file

            if not file_path.is_file():
                raise FileNotFoundError(ini_file)

        logging.debug(f"using config file {str(ini_file)}")
        self._config_dir = file_path.parent
        parser = configparser.ConfigParser()
        parser.read(str(file_path))
        if parser.has_section("tlsmate"):
            self._config_section = parser["tlsmate"]

    def _cast_item(self, item: str, val: Any) -> Any:
        """Cast the type of a configuration item into the internal format."""

        item_type = self._descr[item].type
        if item_type in self._format_option:
            val = self._format_option[item_type](self, val)
        return val

    def _determine_value(self, item: str) -> Any:
        val = os.environ.get("TLSMATE_" + item.upper())
        if val is None and self._config_section:
            val = self._config_section.get(item)  # type: ignore

        if val is not None:
            val = self._cast_item(item, val)

        return val

    def init_from_external(self, ini_file: Optional[Union[str, Path]] = None):
        """Take the configuration from the ini file and from the environment variables.

        Arguments:
            ini_file (str): the path to the ini file. If None is given, then only
                the environment variables are taken into account.
        """

        self._init_config_section(ini_file)
        for item in self._config:
            val = self._determine_value(item)
            if val is not None:
                self._config[item] = val

    def get_from_external(self, ini_file: Optional[str], item: str) -> Any:
        """Gets the value for a single itemerving the ini-file and the environment."""

        self._init_config_section(ini_file)
        return self._determine_value(item)

    def items(self) -> ItemsView[str, structs.ConfigItem]:
        """Return the configuration items. Mimics the dict's `items` method.

        Returns:
            The list of configuration items. Each item is a tuple of the item
            name and the item value.

        """

        return self._config.items()

    def get(self, key: str, default: Any = None) -> Any:
        """Get the value of a configuration item. Mimics the dict's `get` method.

        Arguments:
            key: the name of the configuration item
            default: the default value to return in case the configuration item is
                not existing. Defaults to None.

        Returns:
            the value of the configuration item or the provided default value if
            it is not present.
        """

        return self._config.get(key, default)

    def set(self, key: str, val: Any, keep_existing: bool = True) -> None:
        """Add a configuration option.

        Arguments:
            key: the name of the option
            val: the value of the option
            keep_existing: if set to True and the value is None, an existing
                configuration will not be overwritten. Defaults to True
        """

        if key in self._config and val is None and keep_existing:
            return

        self._config[key] = val

    def register(self, config_item: structs.ConfigItem) -> None:
        """Add a new item to the configuration

        Arguments:
            config_item: the configuration item to register

        Raises:
            ValueError: if a configuration item with the same name is already existing
        """

        name = config_item.name
        if name in self._descr:
            if config_item != self._descr[name]:
                raise ValueError(
                    f'configuration setting "{name}" already defined with '
                    f"different properties"
                )
        self._descr[name] = config_item
        self._config[name] = config_item.default



# -*- coding: utf-8 -*-
"""Module scanning for the excrypt-then-mac extension
"""
# import basic stuff

# import own stuff
import tlsmate.plugin as plg
import tlsmate.tls as tls
import tlsmate.utils as utils

# import other stuff


class ScanEncryptThenMac(plg.Worker):
    name = "encrypt_then_mac"
    descr = "scan for extension encrypt_then_mac support"
    prio = 30

    def run(self):
        state = tls.ScanState.UNDETERMINED
        versions = [tls.Version.TLS10, tls.Version.TLS11, tls.Version.TLS12]

        prof_values = self.server_profile.get_profile_values(versions, full_hs=True)
        cipher_suites = utils.filter_cipher_suites(
            prof_values.cipher_suites, cipher_type=[tls.CipherType.BLOCK]
        )

        if not cipher_suites:
            # no CBC cipher suite supported
            state = tls.ScanState.NA
        else:
            self.client.init_profile(profile_values=prof_values)
            self.client.profile.cipher_suites = cipher_suites
            self.client.profile.support_encrypt_then_mac = True
            with self.client.create_connection() as conn:
                conn.handshake()
            if conn.handshake_completed:
                if conn.msg.server_hello.get_extension(tls.Extension.ENCRYPT_THEN_MAC):
                    state = tls.ScanState.TRUE
                else:
                    state = tls.ScanState.FALSE

        self.server_profile.allocate_features()
        self.server_profile.features.encrypt_then_mac = state



# -*- coding: utf-8 -*-
"""Module implementing helper functions for handling protocol data units
"""
# import basic stuff
import struct
from typing import Tuple

# import own stuff
import tlsmate.tls as tls

# import other stuff


def pack_uint8(val: int) -> bytes:
    """Packs a uint8 value into a byte object.

    Arguments:
        val: The value to pack.

    Returns:
        The bytes object representing the value in network order (big endian).

    Raises:
        ValueError: If the value cannot be represented in one byte.
    """

    if val > 0xFF:
        raise ValueError(f"Cannot pack {val} into 1 byte")

    return struct.pack("!B", val)


def pack_uint16(val: int) -> bytes:
    """Packs a uint16 value into a byte object.

    Arguments:
        val: The value to pack.

    Returns:
        The bytes object representing the value in network order (big endian).

    Raises:
        ValueError: If the value cannot be represented in two bytes.
    """

    if val > 0xFFFF:
        raise ValueError(f"Cannot pack {val} into 2 bytes")

    return struct.pack("!H", val)


def pack_uint24(val: int) -> bytes:
    """Packs a uint24 value into a byte object.

    Arguments:
        val: The value to pack.

    Returns:
        The bytes object representing the value in network order (big endian).

    Raises:
        ValueError: If the value cannot be represented in three bytes.
    """

    if val > 0xFFFFFF:
        raise ValueError(f"Cannot pack {val} into 3 bytes")

    return struct.pack("!I", val)[1:]


def pack_uint32(val: int) -> bytes:
    """Packs a uint32 value into a byte object.

    Arguments:
        val: The value to pack.

    Returns:
        The bytes object representing the value in network order (big endian).

    Raises:
        ValueError: If the value cannot be represented in four bytes.
    """

    if val > 0xFFFFFFFF:
        raise ValueError(f"Cannot pack {val} into 4 bytes")

    return struct.pack("!I", val)


def pack_uint64(val: int) -> bytes:
    """Packs a uint64 value into a byte object.

    Arguments:
        val: The value to pack.

    Returns:
        The bytes object representing the value in network order (big endian).

    Raises:
        ValueError: If the value cannot be represented in eight bytes.
    """

    if val > 0xFFFFFFFFFFFFFFFF:
        raise ValueError(f"Cannot pack {val} into 8 bytes")

    return struct.pack("!Q", val)


def pack_str(string: str) -> bytes:
    """Packs a string into a byte object.

    Arguments:
        string: The string to pack.

    Returns:
        The bytes object representing the string in network order (big endian).
    """

    return bytes(map(ord, string))


def unpack_uint8(data: bytes, offset: int) -> Tuple[int, int]:
    """Unpacks a value from a given an buffer.

    Arguments:
        data: The buffer to unpack from.
        offset: The offset within the buffer.

    Returns:
        The tuple with the result. The first entry is the value that has been
        unpacked, the second one is the new offset in the buffer, i.e. it
        points to the next byte after the unpacked value.

    Raises:
        :obj:`tlsmate.tls.ServerMalfunction`: If the buffer boundary is exceeded.
    """

    if offset >= len(data):
        raise tls.ServerMalfunction(tls.ServerIssue.PARAMETER_LENGTH_ERROR)

    return data[offset], offset + 1


def unpack_uint16(data: bytes, offset: int) -> Tuple[int, int]:
    """Unpacks a value from a given an buffer.

    Arguments:
        data: The buffer to unpack from.
        offset: The offset within the buffer.

    Returns:
        The tuple with the result. The first entry is the value that has been
        unpacked, the second one is the new offset in the buffer, i.e. it
        points to the next byte after the unpacked value.

    Raises:
        :obj:`tlsmate.tls.ServerMalfunction`: If the buffer boundary is exceeded.
    """

    if offset + 1 >= len(data):
        raise tls.ServerMalfunction(tls.ServerIssue.PARAMETER_LENGTH_ERROR)

    return struct.unpack("!H", data[offset : offset + 2])[0], offset + 2


def unpack_uint24(data: bytes, offset: int) -> Tuple[int, int]:
    """Unpacks a value from a given an buffer.

    Arguments:
        data: The buffer to unpack from.
        offset: The offset within the buffer.

    Returns:
        The tuple with the result. The first entry is the value that has been
        unpacked, the second one is the new offset in the buffer, i.e. it
        points to the next byte after the unpacked value.

    Raises:
        :obj:`tlsmate.tls.ServerMalfunction`: If the buffer boundary is exceeded.
    """

    if offset + 2 >= len(data):
        raise tls.ServerMalfunction(tls.ServerIssue.PARAMETER_LENGTH_ERROR)

    high_byte, val = struct.unpack("!BH", data[offset : offset + 3])
    return 0x10000 * high_byte + val, offset + 3


def unpack_uint32(data: bytes, offset: int) -> Tuple[int, int]:
    """Unpacks a value from a given an buffer.

    Arguments:
        data: The buffer to unpack from.
        offset: The offset within the buffer.

    Returns:
        The tuple with the result. The first entry is the value that has been
        unpacked, the second one is the new offset in the buffer, i.e. it
        points to the next byte after the unpacked value.

    Raises:
        :obj:`tlsmate.tls.ServerMalfunction`: If the buffer boundary is exceeded.
    """

    if offset + 3 >= len(data):
        raise tls.ServerMalfunction(tls.ServerIssue.PARAMETER_LENGTH_ERROR)

    return struct.unpack("!I", data[offset : offset + 4])[0], offset + 4


def unpack_bytes(data: bytes, offset: int, length: int) -> Tuple[bytes, int]:
    """Unpacks a given number of bytes from a given an buffer.

    Arguments:
        data: The buffer to unpack from.
        offset: The offset within the buffer.
        length: The number of bytes to unpack.

    Returns:
        The tuple with the result. The first entry is the value that has been
        unpacked, the second one is the new offset in the buffer, i.e. it
        points to the next byte after the unpacked value.

    Raises:
        :obj:`tlsmate.tls.ServerMalfunction`: If the buffer boundary is exceeded.
    """

    if offset + length > len(data):
        raise tls.ServerMalfunction(tls.ServerIssue.PARAMETER_LENGTH_ERROR)

    return data[offset : offset + length], offset + length


def dump(data: bytes, separator: str = " ", with_length: bool = True) -> str:
    """Provide a human readable representation of a bytes object.

    Arguments:
        data: The data to represent
        separator: the separator character(s) between the bytes. Defaults to " ".
        with_length: indication, if the length shall be appended in brackets.

    Returns:
        A human readable string, with a blank between each byte, and the length
        of the string appended in brackets.
    """

    ret = separator.join(f"{y:02x}" for y in data)
    if with_length:
        ret = ret + f" ({len(data)})"

    return ret


def dump_short(
    data: bytes,
    separator: str = " ",
    with_length: bool = True,
    start: int = 10,
    end: int = 10,
) -> str:
    """Like dump, but shorten the hexdump if it exceeds a certain limit.

        If the data is longer than start + end, then only a part of the data will
        be returned, and the middle of the data is collapsed to " ... ".

    Arguments:
        data: The data to represent
        separator: the separator character(s) between the bytes. Defaults to " ".
        with_length: indication, if the length shall be appended in brackets.
        start: the first number of bytes to display
        end: the last number of bytes to display.

    Returns:
        A human readable string, with a blank between each byte, and the length
        of the string appended in brackets.
    """

    length = len(data)
    if length <= start + end:
        return dump(data, separator, with_length)

    else:
        ret = (
            f"{dump(data[:start], separator=separator, with_length=False)} ... "
            f"{dump(data[-end:], separator=separator, with_length=False)}"
        )
        if with_length:
            ret += f" ({length})"

        return ret


def string(data: bytes) -> str:
    """Simple version of dump, separating the data with a colon, and omit the length

    Arguments:
        data (bytes): The data to represent

    Returns:
        str: A human readable string, with a colon between each byte.
    """

    return dump(data, separator=":", with_length=False)



# -*- coding: utf-8 -*-
"""Test connection to external server.
"""
import tlsmate.tls as tls


def test_main(tlsmate):
    tlsmate.client.set_profile(tls.Profile.INTEROPERABILITY)
    with tlsmate.client.create_connection("mozilla-modern.badssl.com") as conn:
        conn.handshake()

    assert conn.handshake_completed



# -*- coding: utf-8 -*-
"""Implements a class to test the ocsp-stapling worker.
"""
import pathlib
from tlsmate.workers.ocsp_stapling import ScanOcspStapling
from tlsmate.tlssuite import TlsSuiteTester
from tlsmate.tlssuite import TlsLibrary


class TestCase(TlsSuiteTester):
    """Class used for tests with pytest.

    For more information refer to the documentation of the TcRecorder class.
    """

    sp_in_yaml = "profile_sig_algos_openssl3_0_0"
    recorder_yaml = "recorder_ocsp_stapling_tls13_not_supported"
    path = pathlib.Path(__file__)
    server_cmd = (
        "utils/start_openssl --version {library} --port {server_port} "
        "--cert1 server-rsa --cert2 server-ecdsa --ca-file ca-certificates "
        "-- -www -cipher ALL"
    )
    library = TlsLibrary.openssl3_0_0

    server = "localhost"

    def run(self, tlsmate, is_replaying):
        server_profile = tlsmate.server_profile
        ScanOcspStapling(tlsmate).run()
        profile = server_profile.make_serializable()
        assert profile["features"]["ocsp_stapling"] == "FALSE"
        assert profile["features"]["ocsp_multi_stapling"] == "FALSE"


if __name__ == "__main__":
    TestCase().entry(is_replaying=False)



# -*- coding: utf-8 -*-
"""Implements a class to be used for unit testing.
"""
import pathlib
import logging
from tests.cipher_suite_tester import CipherSuiteTester
from tlsmate import tls
from tlsmate import msg
from tlsmate.tlssuite import TlsLibrary


class TestCase(CipherSuiteTester):
    """Class used for tests with pytest.

    For more information refer to the documentation of the CipherSuiteTester class.
    """

    path = pathlib.Path(__file__)

    cipher_suite = tls.CipherSuite.TLS_CHACHA20_POLY1305_SHA256
    server_cmd = (
        "utils/start_openssl --version {library} --port {server_port} "
        "--cert1 server-rsa --cert2 server-ecdsa "
        "-- -www -cipher ALL"
    )
    library = TlsLibrary.openssl1_1_1

    # Uncomment the line below if you do not want to use the default version and
    # adapt it to your needs.
    # version = tls.Version.TLS12

    def run(self, tlsmate, is_replaying=False):
        client = tlsmate.client
        client.init_profile()

        client.profile.versions = [tls.Version.TLS13]
        client.profile.cipher_suites = [tls.CipherSuite.TLS_CHACHA20_POLY1305_SHA256]
        client.profile.supported_groups = [tls.SupportedGroups.SECP256R1]
        client.profile.key_shares = [tls.SupportedGroups.SECP256R1]
        client.profile.signature_algorithms = [
            tls.SignatureScheme.RSA_PKCS1_SHA1,
            tls.SignatureScheme.ECDSA_SHA1,
            tls.SignatureScheme.RSA_PKCS1_SHA256,
            tls.SignatureScheme.ECDSA_SECP256R1_SHA256,
            tls.SignatureScheme.RSA_PKCS1_SHA384,
            tls.SignatureScheme.ECDSA_SECP384R1_SHA384,
            tls.SignatureScheme.RSA_PKCS1_SHA512,
            tls.SignatureScheme.ECDSA_SECP521R1_SHA512,
            tls.SignatureScheme.ED25519,
            tls.SignatureScheme.ED448,
            tls.SignatureScheme.RSA_PSS_PSS_SHA256,
            tls.SignatureScheme.RSA_PSS_PSS_SHA384,
            tls.SignatureScheme.RSA_PSS_PSS_SHA512,
        ]

        client.profile.support_session_ticket = True
        end_of_tc_reached = False
        with client.create_connection() as conn:

            conn.send(msg.ClientHello)
            conn.wait(msg.ServerHello)
            conn.wait(msg.ChangeCipherSpec, optional=True)
            conn.wait(msg.EncryptedExtensions)
            conn.wait(msg.Certificate, optional=True)
            conn.wait(msg.CertificateVerify, optional=True)
            conn.wait(msg.Finished)
            conn.send(msg.Finished)
            conn.wait(msg.NewSessionTicket)
            conn.wait(msg.NewSessionTicket)

            conn.send(msg.AppData(b"GET / HTTP/1.1\n"))
            app_data = conn.wait(msg.AppData)
            while not len(app_data.data):
                app_data = conn.wait(msg.AppData)
            for line in app_data.data.decode("utf-8").split("\n"):
                if line.startswith("s_server"):
                    logging.debug("openssl_command: " + line)
                    conn.recorder.trace(openssl_command=line)
            end_of_tc_reached = True

        assert end_of_tc_reached is True


if __name__ == "__main__":
    TestCase().entry(is_replaying=False)



# -*- coding: utf-8 -*-
"""Implement unit tests for the module pdu.
"""
import tlsmate.tls as tls
from tlsmate import pdu
import pytest


def test_pack_uint8_overflow():
    val = 0xFF
    assert pdu.pack_uint8(val) == bytes.fromhex("ff")
    with pytest.raises(ValueError, match=f"Cannot pack {val + 1} into 1 byte"):
        pdu.pack_uint8(val + 1)


def test_pack_uint16_overflow():
    val = 0xFFFF
    assert pdu.pack_uint16(val) == bytes.fromhex("ff ff")
    with pytest.raises(ValueError, match=f"Cannot pack {val +1 } into 2 bytes"):
        pdu.pack_uint16(val + 1)


def test_pack_uint24_overflow():
    val = 0xFFFFFF
    assert pdu.pack_uint24(val) == bytes.fromhex("ff ff ff")
    with pytest.raises(ValueError, match=f"Cannot pack {val +1 } into 3 bytes"):
        pdu.pack_uint24(val + 1)


def test_pack_uint32_overflow():
    val = 0xFFFFFFFF
    assert pdu.pack_uint32(val) == bytes.fromhex("ff ff ff ff")
    with pytest.raises(ValueError, match=f"Cannot pack {val +1 } into 4 bytes"):
        pdu.pack_uint32(val + 1)


def test_pack_uint64_overflow():
    val = 0xFFFFFFFFFFFFFFFF
    assert pdu.pack_uint64(val) == bytes.fromhex("ff ff ff ff ff ff ff ff")
    with pytest.raises(ValueError, match=f"Cannot pack {val +1 } into 8 bytes"):
        pdu.pack_uint64(val + 1)


def test_unpack_uint8():
    data = bytes.fromhex("88 77 66 55 44 33 22 11")
    val, offset = pdu.unpack_uint8(data, len(data) - 1)
    assert val == 0x11
    with pytest.raises(
        tls.ServerMalfunction, match="message length error when unpacking parameter"
    ):
        val, offset = pdu.unpack_uint8(data, len(data))


def test_unpack_uint16():
    data = bytes.fromhex("88 77 66 55 44 33 22 11")
    val, offset = pdu.unpack_uint16(data, len(data) - 2)
    assert val == 0x2211
    with pytest.raises(
        tls.ServerMalfunction, match="message length error when unpacking parameter"
    ):
        val, offset = pdu.unpack_uint16(data, len(data) - 1)


def test_unpack_uint24():
    data = bytes.fromhex("88 77 66 55 44 33 22 11")
    val, offset = pdu.unpack_uint24(data, len(data) - 3)
    assert val == 0x332211
    with pytest.raises(
        tls.ServerMalfunction, match="message length error when unpacking parameter"
    ):
        val, offset = pdu.unpack_uint24(data, len(data) - 2)


def test_unpack_uint32():
    data = bytes.fromhex("88 77 66 55 44 33 22 11")
    val, offset = pdu.unpack_uint32(data, len(data) - 4)
    assert val == 0x44332211
    with pytest.raises(
        tls.ServerMalfunction, match="message length error when unpacking parameter"
    ):
        val, offset = pdu.unpack_uint32(data, len(data) - 3)


def test_unpack_bytes():
    data = bytes.fromhex("88 77 66 55 44 33 22 11")
    val, offset = pdu.unpack_bytes(data, 0, len(data))
    assert val == data
    with pytest.raises(
        tls.ServerMalfunction, match="message length error when unpacking parameter"
    ):
        val, offset = pdu.unpack_bytes(data, 0, len(data) + 1)



# -*- coding: utf-8 -*-
"""Module with a class for recording a tls connection
"""
# import basic stuff
import enum
import datetime
import time
from typing import List, Any, Dict, Tuple, Optional

# import own stuff
import tlsmate.config as conf
import tlsmate.tls as tls
import tlsmate.utils as utils


# import other stuff
import requests


class RecorderState(enum.Enum):
    """States for the recorder
    """

    INACTIVE = 0
    RECORDING = 1
    REPLAYING = 2


class SocketEvent(enum.Enum):
    """Events that might occur when waiting for socket data
    """

    TIMEOUT = 0
    DATA = 1
    CLOSURE = 2


class Recorder(object):
    """Class implementing a recorder mechanism.

    The purpose of the class is to have a build-in mechanism for unit testing. It works
    by providing hooks to all external interfaces. `External interface` means messages
    sent and received via the socket as well as numbers generated randomly.

    If inactive, the recorder has no functional impact on the rest of tlsmate.
    When recording, the recorder stores all data which are passing the interfaces,
    and they are `recorded`, i.e., they are stored in the recorder object. This mode
    is used to record a unit test case (which can be as complex as a complete scan of
    a server). After the recoding is finished, the complete recorder object is
    serialized to a YAML file.
    When replaying (normally triggered by pytest), the recorder object is deserialized
    from the file, and all recorded data is injected when the external interfaces
    are used. This way an EXACT clone of the connection(s) is/are executed. The
    replayed test case uses the same keying material as well, it is a `byte-to-byte`
    copy. Of course, all data sent over external interfaces are checked, and any
    deviation with the previously recorded data will let the test case fail.
    Note, that even after some cryptographic operations the recorder is hooked in, this
    allows easier debugging in case a replayed test deviates from the recorded twin.
    """

    _attr = {
        "client_random": bytes,
        "server_random": bytes,
        "pre_master_secret": bytes,
        "master_secret": bytes,
        "private_key": bytes,
        "client_write_mac_key": bytes,
        "server_write_mac_key": bytes,
        "client_write_key": bytes,
        "server_write_key": bytes,
        "client_write_iv": bytes,
        "server_write_iv": bytes,
        "verify_data_finished_rec": bytes,
        "verify_data_finished_calc": bytes,
        "msg_digest_finished_rec": bytes,
        "msg_digest_finished_sent": bytes,
        "verify_data_finished_sent": bytes,
        "ec_seed": int,
        "pms_rsa": bytes,
        "rsa_enciphered": bytes,
        "x_val": bytes,
        "y_val": int,
        "openssl_command": str,
        "early_secret": bytes,
        "early_tr_secret": bytes,
        "msg_digest_tls13": bytes,
        "binder": bytes,
        "binder_key": bytes,
        "finished_key": bytes,
        "msg_without_binders": bytes,
        "hmac_algo": str,
        "timestamp": float,
        "datetime": datetime.datetime,
        "msg_sendall": bytes,
        "msg_recv": "msg_recv",
        "crl_url": str,
        "crl": bytes,
        "signature": bytes,
        "trust_store": str,
        "client_auth": bool,
        "client_key_chain": "client_key_chain",
        "grease": int,
        "response": "response",
    }

    def __init__(self, config: Optional[conf.Configuration] = None) -> None:
        self.reset()
        self._delay = True
        if config:
            self._delay = config.get("recorder_delay")

    def reset(self) -> None:
        """Reset the recorder to an initial state.
        """
        self._state = RecorderState.INACTIVE
        self._add_delay = None
        self.data: Dict[str, Any] = {}
        for key in self._attr.keys():
            self.data[key] = []

    @staticmethod
    def _serialize_val(val, val_type):
        """Convert values to JSON/YAML serializable types

        Arguments:
            val: the value to convert
            val_type: the type of value

        Returns:
            ret: the converted value
        """

        if val_type is bytes:
            return val.hex()

        elif val_type is datetime.datetime:
            return val.timestamp()

        elif val_type == "msg_recv":
            timeout, event_type, data = val
            if data is not None:
                data = data.hex()

            return [timeout, event_type.value, data]

        elif val_type == "response":
            timeout, event_type, data = val
            if data is not None:
                data = [data.ok, data.content.hex(), data.status_code]

            return [timeout, event_type.value, data]

        return val

    @staticmethod
    def _deserialize_val(val, val_type):
        """Convert a serialized type to the original type

        Arguments:
            val: the value to convert
            val_type: the type of value

        Returns:
            ret: the converted value
        """

        if val_type is bytes:
            return bytes.fromhex(val)

        elif val_type is datetime.datetime:
            return datetime.datetime.fromtimestamp(val)

        elif val_type == "msg_recv":
            timout, event_type, data = val
            event_type = SocketEvent(event_type)
            if data is not None:
                data = bytes.fromhex(data)

            val = (timout, event_type, data)

        elif val_type == "response":
            timout, event_type, data = val
            event_type = SocketEvent(event_type)
            if data is not None:
                data[1] = bytes.fromhex(data[1])
            val = (timout, event_type, data)

        return val

    def _store_value(self, name, value):
        self.data[name].append(self._serialize_val(value, self._attr[name]))

    def _unstore_value(self, name):
        if not self.data[name]:
            return None

        return self._deserialize_val(self.data[name].pop(0), self._attr[name])

    def deactivate(self) -> None:
        """Deactivate the recorder.
        """

        self._state = RecorderState.INACTIVE

    def record(self) -> None:
        """Activate the recorder to record a test case.
        """

        self._state = RecorderState.RECORDING

    def replay(self) -> None:
        """Activate the recorder to replay a test case.
        """

        self._state = RecorderState.REPLAYING

    def is_injecting(self) -> bool:
        """Check if the recorder is currently replaying (i.e. injecting data)

        Returns:
            True if the recorder is replaying
        """

        return self._state is RecorderState.REPLAYING

    def is_recording(self) -> bool:
        """Check if the recorder is currently recording.

        Returns:
            True, if the recorder is recording
        """

        return self._state is RecorderState.RECORDING

    def get_trust_store(self) -> List[str]:
        """Provide the data for the trust store

        Returns:
            A list of certificates in the trust store. These are strings
            representing the certificates in DER-format.
        """

        return self.data["trust_store"]

    def trace_client_auth(self, cl_auth: Tuple[str, List[str]]) -> None:
        """Add a set of client authentication data to the recorder.

        Arguments:
            cl_auth: the key and the certificate chain to add to the recorder.
                Both must be provided as strings.
        """

        if self._state is RecorderState.RECORDING:
            self.data["client_key_chain"].append(cl_auth)

    def get_client_auth(self) -> List[Tuple[bytes, bytes]]:
        """Provide access to a list of client authentication sets.

        Returns:
            list: A list of (key/cert-chain) pairs.
        """

        return self.data["client_key_chain"]

    def trace_socket_recv(
        self, timeout: float, event_type: SocketEvent, data: Optional[bytes] = None
    ) -> None:
        """Trace a message received from a socket (if state is recording).

        Arguments:
            timeout: the timeout after that the event occurred in seconds
            event_type: the event that occurred
            data: the message in raw format (if event_type is data)
        """

        if self._state is RecorderState.RECORDING:
            if self._add_delay is not None:
                timeout += self._add_delay
                self._add_delay = None

            self._store_value("msg_recv", (timeout, event_type, data))

    def inject_socket_recv(self) -> Optional[bytes]:
        """If the recorder is replaying, inject the previously recorded message.

        Returns:
            the message previously recorded
        """

        if self._state is RecorderState.REPLAYING:
            timeout, event_type, data = self._unstore_value("msg_recv")
            if self._delay:
                time.sleep(timeout)

            if event_type is SocketEvent.CLOSURE:
                raise tls.TlsConnectionClosedError

            elif event_type is SocketEvent.TIMEOUT:
                raise tls.TlsMsgTimeoutError

            return data

        return None

    def additional_delay(self, delay: float) -> None:
        """Indicate that there is an additional delay when waiting to inject a message.

        Arguments:
            delay: The additional delay to take into account when injecting a
                received message the next time.
        """

        if self._add_delay is None:
            self._add_delay = delay  # type: ignore

        else:
            self._add_delay += delay

    def trace_socket_sendall(self, msg: bytes) -> bool:
        """Interface for the sendall socket function.

        Arguments:
            msg: the message to send over the socket

        Returns:
            True, if the message is sent externally.
        """

        if self._state is RecorderState.RECORDING:
            self._store_value("msg_sendall", msg)

        return self._state != RecorderState.REPLAYING

    def trace(self, **kwargs: Any) -> None:
        """Interface to trace any data

        Arguments:
            **kwargs: the name and value to trace
        """

        if self._state is RecorderState.INACTIVE:
            return

        name, val = kwargs.popitem()
        if name in self._attr:
            if self._state is RecorderState.REPLAYING:
                assert val == self._unstore_value(name)

            else:
                self._store_value(name, val)

    def inject(self, **kwargs: Any) -> Any:
        """Interface to potentially inject recorded data

        If recording, the data provided is stored in the recorder object.
        If replaying, the data previously recorded is returned.

        Arguments:
            **kwargs: the name and value of the data

        Returns:
            If recording: the data provided via kwargs, if replaying: the data
            previously recorded.
        """

        name, val = kwargs.popitem()
        if self._state is RecorderState.INACTIVE:
            return val

        if name in self._attr:
            if self._state is RecorderState.REPLAYING:
                val = self._unstore_value(name)

            else:
                self._store_value(name, val)

        return val

    def inject_response(self) -> Optional[Any]:
        """Inject a :obj:`requests.Response` object

        Only minimal ducktyping is supported.

        Returns:
            an object with the attributes ok, status_code and content
        """

        class _Response:
            pass

        if self._state is RecorderState.REPLAYING:
            timeout, event_type, data = self._unstore_value("response")
            if self._delay:
                time.sleep(timeout)

            if event_type is SocketEvent.CLOSURE:
                raise Exception

            elif event_type is SocketEvent.TIMEOUT:
                raise requests.Timeout

            response = _Response()
            response.ok, response.content, response.status_code = data  # type: ignore
            return response

        return None

    def trace_response(
        self, timeout: float, event_type: SocketEvent, data: Optional[bytes] = None
    ) -> None:
        """Trace the result of a post/get/... request.

        Arguments:
            timeout: the timeout after which the response was received.
            event_type: the event that occurred
            data: the response object from which only the most relevant
                attributes are recorded (ok, status_code, content). Only
                present if event_type is SocketEvent.DATA.
        """

        if self._state is RecorderState.RECORDING:
            self._store_value("response", (timeout, event_type, data))

    def serialize(self, filename: str) -> None:
        """Serialize the recorded data to a file using YAML

        Arguments:
            filename: the file name to store the data in
        """

        utils.serialize_data(self.data, file_name=filename, replace=False, indent=2)

    def deserialize(self, filename: str) -> None:
        """Deserialize the recorded data from a file

        Arguments:
            filename (str): the file name to deserialize the data from
        """

        self.data = utils.deserialize_data(filename)

    def get_timestamp(self) -> datetime.datetime:
        """Return an appropriate timestamp

        Returns:
            datetime.datetime: the timestamp. If the recorder is injecting, it is the
            recorded timestamp, else the current time (which is recorded, if the
            recorder is active).
        """
        if self.is_injecting():
            return self.inject(datetime=None)

        else:
            timestamp = datetime.datetime.now()
            self.trace(datetime=timestamp)
            return timestamp



# -*- coding: utf-8 -*-
"""Module scanning for TLS protocol versions, cipher suites and certificate chains
"""
# import basic stuff
import logging

# import own stuff
import tlsmate.msg as msg
import tlsmate.plugin as plg
import tlsmate.server_profile as server_profile
import tlsmate.tls as tls
import tlsmate.utils as utils

# import other stuff


class ScanCipherSuites(plg.Worker):
    """Scans for the supported versions, cipher suites and certificate chains.

    The results are stored in the server profile.
    """

    name = "cipher_suites"
    descr = "scan for supported TLS versions, cipher suites and certificates"
    prio = 10

    config_mapping = {
        tls.Version.SSL20: "sslv2",
        tls.Version.SSL30: "sslv3",
        tls.Version.TLS10: "tls10",
        tls.Version.TLS11: "tls11",
        tls.Version.TLS12: "tls12",
        tls.Version.TLS13: "tls13",
    }

    supported_groups_tls12 = [
        tls.SupportedGroups.X25519,
        tls.SupportedGroups.X448,
        tls.SupportedGroups.SECT163K1,
        tls.SupportedGroups.SECT163R2,
        tls.SupportedGroups.SECT233K1,
        tls.SupportedGroups.SECT233R1,
        tls.SupportedGroups.SECT283K1,
        tls.SupportedGroups.SECT283R1,
        tls.SupportedGroups.SECT409K1,
        tls.SupportedGroups.SECT409R1,
        tls.SupportedGroups.SECT571K1,
        tls.SupportedGroups.SECT571R1,
        tls.SupportedGroups.SECP224R1,
        tls.SupportedGroups.SECP256K1,
        tls.SupportedGroups.BRAINPOOLP256R1,
        tls.SupportedGroups.BRAINPOOLP384R1,
        tls.SupportedGroups.BRAINPOOLP512R1,
        tls.SupportedGroups.SECP256R1,
        tls.SupportedGroups.SECP384R1,
        tls.SupportedGroups.SECP521R1,
        tls.SupportedGroups.FFDHE2048,
        tls.SupportedGroups.FFDHE4096,
    ]

    def _get_server_cs_and_cert(self, version):
        """Performs a handshake and retieves the cipher suite and certificate chain.

        The certificate chain received from the server is added to the server profile.

        Arguments:
            version (:obj:`tlsmate.tls.Version`): The TLS version

        Returns:
            :obj:`tlsmate.tls.CipherSuite`: the cipher suite selected by the server.
                None, if no SeverHello was received.
        """
        with self.client.create_connection() as conn:
            conn.send(msg.ClientHello)
            server_hello = conn.wait(msg.ServerHello)
            if server_hello is None:
                return None

            if version is not server_hello.get_version():
                return None

            if version is tls.Version.TLS13:
                conn.wait(msg.ChangeCipherSpec, optional=True)
                conn.wait(msg.EncryptedExtensions)

            certificate = conn.wait(msg.Certificate, optional=True)
            if certificate is not None:
                self.server_profile.append_unique_cert_chain(certificate.chain)

            return server_hello.cipher_suite

        return None

    def _get_server_cs(self):
        """Performs a handshake and returns the ciper suite selected by the server.

        Returns:
            :obj:`tlsmate.tls.CipherSuite`: the cipher suite selected by the server.
                None, if no SeverHello was received.
        """
        with self.client.create_connection() as conn:
            conn.send(msg.ClientHello)
            server_hello = conn.wait(msg.Any)

        try:
            return server_hello.cipher_suite

        except AttributeError:
            return None

    def _get_server_preference(self, cipher_suites):
        """Determine the server preference order of supported cipher suites.

        Arguments:
            cipher_suites (list of :obj:`tlsmate.tls.CipherSuite`): the list of
                cipher suites support by the server in arbitrary order.

        Returns:
            list of :obj:`tlsmate.tls.CipherSuite`: the cipher suites supported by
                the server in the order of preference.
        """
        self.client.profile.cipher_suites = cipher_suites
        server_pref = []
        while self.client.profile.cipher_suites:
            server_cs = self._get_server_cs()
            server_pref.append(server_cs)
            self.client.profile.cipher_suites.remove(server_cs)

        return server_pref

    def _chacha_poly_pref(self, server_pref, ciphers):
        if server_pref is not tls.ScanState.TRUE:
            return tls.ScanState.NA

        tmp_cs = ciphers.cipher_suites[:]
        chacha_cs = utils.filter_cipher_suites(
            tmp_cs, cipher_prim=[tls.CipherPrimitive.CHACHA], remove=True
        )
        if not chacha_cs:
            return tls.ScanState.NA

        check_chacha_pref = False
        for idx, cs in enumerate(ciphers.cipher_suites):
            if cs not in chacha_cs:
                if idx < len(chacha_cs):
                    check_chacha_pref = True

                break

        if not check_chacha_pref:
            return tls.ScanState.NA

        chacha_cs.extend(tmp_cs)
        self.client.profile.cipher_suites = chacha_cs
        with self.client.create_connection() as conn:
            conn.send(msg.ClientHello)
            server_hello = conn.wait(msg.ServerHello)
            if server_hello.cipher_suite in chacha_cs:
                return tls.ScanState.TRUE

            else:
                return tls.ScanState.FALSE

        return tls.ScanState.UNDETERMINED

    def _tls_enum_version(self, version, vers_prof):
        """Determines the supported cipher suites and other stuff for a given version.

        Determines the supported cipher suites, if the server enforces the priority
        (and if so, determines the order of cipher suites according to their priority)
        and extracts the certificate chains and stores them in the server profile.

        Arguments:
            version (:obj:`tlsmate.tls.Version`): the TLS version to enumerate.
            vers_prof (:obj:`tlsmate.server_profile.SPVersion`): the version profile
        """

        if version is tls.Version.TLS13:
            self.client.profile.supported_groups = tls.SupportedGroups.all_tls13()

        else:
            self.client.profile.supported_groups = self.supported_groups_tls12

        cipher_suites = utils.filter_cipher_suites(
            tls.CipherSuite.all(), version=version
        )

        if version is tls.Version.TLS12:
            # put all CHACHA_POLY cipher suites at the end, so that there is no
            # interference with potential "chacha_poly_preference".
            chacha_cs = utils.filter_cipher_suites(
                cipher_suites, cipher_prim=[tls.CipherPrimitive.CHACHA], remove=True
            )
            cipher_suites.extend(chacha_cs)

        logging.info(f"starting to enumerate {version.name}")
        self.client.profile.versions = [version]
        supported_cs = []

        # get a list of all supported cipher suites, don't send more than
        # max_items cipher suites in the ClientHello
        max_items = 32
        while cipher_suites:
            sub_set = cipher_suites[:max_items]
            cipher_suites = cipher_suites[max_items:]

            while sub_set:
                self.client.profile.cipher_suites = sub_set
                cipher_suite = self._get_server_cs_and_cert(version)
                if cipher_suite not in (None, tls.CipherSuite.TLS_NULL_WITH_NULL_NULL):
                    sub_set.remove(cipher_suite)
                    supported_cs.append(cipher_suite)

                else:
                    sub_set = []

        if supported_cs:
            if len(supported_cs) == 1:
                server_prio = tls.ScanState.NA

            else:
                server_prio = tls.ScanState.FALSE
                # check if server enforce the cipher suite prio
                self.client.profile.cipher_suites = supported_cs
                if self._get_server_cs() != supported_cs[0]:
                    server_prio = tls.ScanState.TRUE

                else:
                    supported_cs.append(supported_cs.pop(0))
                    if self._get_server_cs() != supported_cs[0]:
                        server_prio = tls.ScanState.TRUE

                # determine the order of cipher suites on server side, if applicable
                if server_prio == tls.ScanState.TRUE:
                    supported_cs = self._get_server_preference(supported_cs)

                else:
                    # esthetical: restore original order, which means the cipher suites
                    # are ordered according to the binary representation
                    supported_cs.insert(0, supported_cs.pop())

            ciphers = server_profile.SPCiphers(
                server_preference=server_prio, cipher_suites=supported_cs
            )
            if version is tls.Version.TLS12:
                ciphers.chacha_poly_preference = self._chacha_poly_pref(
                    server_prio, ciphers
                )

            vers_prof.ciphers = ciphers
            vers_prof.support = tls.ScanState.TRUE

        else:
            vers_prof.support = tls.ScanState.FALSE

        logging.info(f"enumeration for {version} finished")

    def _ssl2_enum_version(self, vers_prof):
        """Minimal support to determine if SSLv2 is supported.
        """
        with self.client.create_connection() as conn:
            conn.send(msg.SSL2ClientHello)
            server_hello = conn.wait(msg.SSL2ServerHello)
            if server_hello is not None:
                vers_prof.cipher_kinds = server_hello.cipher_specs
                vers_prof.server_preference = tls.ScanState.UNDETERMINED
                vers_prof.version = tls.Version.SSL20
                vers_prof.support = tls.ScanState.TRUE
                return

        vers_prof.support = tls.ScanState.FALSE

    def _enum_version(self, version, vers_prof):
        """Scan a specific TLS version.

        Arguments:
            version (:obj:`tlsmate.tls.Version`): The TLS version to enumerate.
        """
        if version is tls.Version.SSL20:
            self._ssl2_enum_version(vers_prof)

        else:
            self._tls_enum_version(version, vers_prof)

    def run(self):
        """The entry point for the worker.
        """

        self.client.alert_on_invalid_cert = False
        self.client.profile.support_extended_master_secret = False
        self.client.profile.support_encrypt_then_mac = False
        self.client.profile.support_session_id = False
        self.client.profile.support_session_ticket = False
        self.client.profile.ec_point_formats = None
        self.client.profile.key_shares = tls.SupportedGroups.all_tls13()
        self.client.profile.signature_algorithms = [
            tls.SignatureScheme.ED25519,
            tls.SignatureScheme.ED448,
            tls.SignatureScheme.ECDSA_SECP384R1_SHA384,
            tls.SignatureScheme.ECDSA_SECP256R1_SHA256,
            tls.SignatureScheme.ECDSA_SECP521R1_SHA512,
            tls.SignatureScheme.RSA_PSS_RSAE_SHA256,
            tls.SignatureScheme.RSA_PSS_RSAE_SHA384,
            tls.SignatureScheme.RSA_PSS_RSAE_SHA512,
            tls.SignatureScheme.RSA_PKCS1_SHA256,
            tls.SignatureScheme.RSA_PKCS1_SHA384,
            tls.SignatureScheme.RSA_PKCS1_SHA512,
            tls.SignatureScheme.ECDSA_SHA1,
            tls.SignatureScheme.RSA_PKCS1_SHA1,
        ]

        self.server_profile.allocate_versions()
        for version in tls.Version.all():
            vers_prof = server_profile.SPVersion(version=version)
            if self.config.get(self.config_mapping[version]):
                self._enum_version(version, vers_prof)

            else:
                vers_prof.support = tls.ScanState.UNDETERMINED

            self.server_profile.versions.append(vers_prof)



# -*- coding: utf-8 -*-
"""Module for dependency injection
"""
# import basic stuff
import pathlib
import logging
from typing import Optional

# import own stuff
import tlsmate.cert as crt
import tlsmate.cert_chain as cert_chain
import tlsmate.client as client
import tlsmate.client_auth as client_auth
import tlsmate.config as conf
import tlsmate.crl_manager as crl_manager
import tlsmate.recorder as rec
import tlsmate.server_profile as server_profile
import tlsmate.trust_store as trust_store

# from tlsmate.plugin import WorkManager
from tlsmate.key_logging import KeyLogger
from tlsmate import utils

# import other stuff

TLSMATE_DIR = pathlib.Path(__file__).parent.parent.resolve()


class TlsMate(object):
    """Class defining the tlsmate application.

    Arguments:
        config (:obj:`tlsmate.config.Configuration`): an object representing tlsmate's
            configuration.

    Attributes:
        config (:obj:`tlsmate.config.Configuration`): the configuration object
            available for most other major object as well
        server_profile (:obj:`tlsmate.server_profile.ServerProfile`): the server
            profile object, describing which features and capabilities are
            supported by the server.
        recorder (:obj:`tlsmate.recorder.Recorder`): the recorder object, available
            for most major objects as well. Used for unit tests only.
        trust_store (:obj:`tlsmate.cert.TrustStore`): the trust store which provides
            access to the root certificates
        crl_manager (:obj:`tlsmate.cert.CrlManager`): the object which manages CRLs.
        client_auth (:obj:`tlsmate.client_auth.ClientAuth`): the object which manages
            the keys and certificate chains needed for client authentication
        client (:obj:`tlsmate.client.Client`): the client object
        cert_chain_cache (:obj:`tlsmate.cert_chain.CertChainCache`): the certificate
            chain validation status cache
    """

    instance = None
    """The instance of tlsmate
    """

    def __init__(self, config: Optional[conf.Configuration] = None) -> None:
        TlsMate.instance = self
        if config is None:
            config = conf.Configuration()

        for key, val in config.items():
            logging.debug(f"using config {key}={val}")

        self.config = config
        self.server_profile = server_profile.ServerProfile()
        self.recorder = rec.Recorder(config=config)
        self.trust_store = trust_store.TrustStore(recorder=self.recorder)
        self.client_auth = client_auth.ClientAuth(recorder=self.recorder)
        self.crl_manager = crl_manager.CrlManager(recorder=self.recorder)
        self.cert_chain_cache = cert_chain.CertChainCache()
        key_log_file = config.get("key_log_file")
        if key_log_file:
            KeyLogger.open_file(key_log_file)

        read_profile = config.get("read_profile")
        if read_profile:
            self.server_profile.load(utils.deserialize_data(read_profile))

        recorder_replaying = config.get("pytest_recorder_replaying")
        if recorder_replaying is not True:
            # "Normal" init of trust store and client auth from configuration
            if recorder_replaying is False:
                self.recorder.record()

            self.trust_store.set_ca_files(config.get("ca_certs"))
            if config.get("client_key"):
                for key_file, chain_file in zip(
                    config.get("client_key"), config.get("client_chain")
                ):
                    self.client_auth.add_auth_files(key_file, chain_file)

            key_log_file = config.get("key_log_file")
            if key_log_file:
                KeyLogger.open_file(key_log_file)

        else:
            pytest_recorder_file = config.get("pytest_recorder_file")
            if pytest_recorder_file:
                self.recorder.deserialize(pytest_recorder_file)

            self.recorder.replay()
            # Init trust store and client auth from recorded data
            for cert in self.recorder.get_trust_store():
                self.trust_store.add_cert(crt.Certificate(der=bytes.fromhex(cert)))

            for key_chain in self.recorder.get_client_auth():
                # TODO: resolve type issue
                self.client_auth.deserialize_key_chain(key_chain)  # type: ignore

        self.client = client.Client(
            config=config, recorder=self.recorder, client_auth=self.client_auth
        )



# -*- coding: utf-8 -*-
"""Implements test cases for the cli
"""
import sys
import pytest
import re

from tlsmate import command
from tlsmate.plugin import WorkManager
from tlsmate.tlsmate import TlsMate
from tlsmate.workers.text_server_profile import TextProfileWorker
from tlsmate.workers.server_profile import DumpProfileWorker, ReadProfileWorker
from tlsmate.workers.compression import ScanCompression
from tlsmate.workers.encrypt_then_mac import ScanEncryptThenMac
from tlsmate.workers.master_secret import ScanExtendedMasterSecret
from tlsmate.workers.resumption import ScanResumption
from tlsmate.workers.renegotiation import ScanRenegotiation
from tlsmate.workers.ccs_injection import ScanCcsInjection
from tlsmate.workers.robot import ScanRobot
from tlsmate.workers.padding_oracle import ScanPaddingOracle
from tlsmate.workers.dh_params import ScanDhGroups
from tlsmate.workers.heartbeat import ScanHeartbeat
from tlsmate.workers.heartbleed import ScanHeartbleed
from tlsmate.workers.grease import ScanGrease
from tlsmate.workers.ephemeral_key_reuse import ScanEphemeralKeyReuse
from tlsmate.workers.ocsp_stapling import ScanOcspStapling
from tlsmate.workers.downgrade import ScanDowngrade
from tlsmate.workers.eval_cipher_suites import ScanCipherSuites
from tlsmate.workers.scanner_info import ScanStart, ScanEnd
from tlsmate.workers.supported_groups import ScanSupportedGroups
from tlsmate.workers.sig_algo import ScanSigAlgs


def work_manager_run(self, tlsmate):
    pass


def test_no_subcommand(capsys):
    cmd = "tlsmate"
    sys.argv = cmd.split()
    with pytest.raises(SystemExit) as pytest_wrapped_e:
        command.main()
    captured = capsys.readouterr()
    assert "tlsmate: error: Subcommand is mandatory" in captured.err
    assert pytest_wrapped_e.value.code == 2


def test_version(capsys):
    cmd = "tlsmate version"
    sys.argv = cmd.split()
    command.main()
    captured = capsys.readouterr()
    assert re.match(r"^\d+\.\d+\.\d+", captured.out)


def test_scan(capsys):
    cmd = "tlsmate scan --port=100000 127.0.0.1"
    sys.argv = cmd.split()
    with pytest.raises(SystemExit) as pytest_wrapped_e:
        command.main()
    captured = capsys.readouterr()
    assert "tlsmate: error: port must be in the range [0-65535]" in captured.err
    assert pytest_wrapped_e.value.code == 2


def test_invalid_domain(capsys):
    cmd = "tlsmate scan --port=0 palimpalim.palimpalim"
    sys.argv = cmd.split()
    with pytest.raises(SystemExit) as pytest_wrapped_e:
        command.main()
    captured = capsys.readouterr()
    assert "Error: Cannot resolve domain name palimpalim.palimpalim" in captured.err
    assert pytest_wrapped_e.value.code == 1


def test_closed_port(capsys):
    cmd = "tlsmate scan --port=0 localhost"
    sys.argv = cmd.split()
    with pytest.raises(SystemExit) as pytest_wrapped_e:
        command.main()
    captured = capsys.readouterr()
    assert "Error: Cannot open TCP connection to TransportEndpoint" in captured.err
    assert pytest_wrapped_e.value.code == 1


def test_all_defaults(monkeypatch, tlsmate_empty_ini):
    monkeypatch.setattr(WorkManager, "run", work_manager_run)

    cmd = f"tlsmate --config {tlsmate_empty_ini} scan 127.0.0.1"
    sys.argv = cmd.split()
    # import pudb; pudb.set_trace()
    command.main()
    config = TlsMate.instance.config
    workers = []
    _ = [workers.extend(pool) for pool in WorkManager._instance._prio_pool.values()]

    assert config.get("port") == 443
    assert config.get("key_log_file") is None
    assert config.get("progress") is False
    assert config.get("sni") is None
    assert config.get("read_profile") is None
    assert config.get("write_profile") is None
    assert config.get("format") == "text"
    assert config.get("color") is True
    assert config.get("ca_certs") is None
    assert config.get("client_key") is None
    assert config.get("client_chain") is None
    assert config.get("crl") is True
    assert config.get("ocsp") is True

    for ver in ["sslv2", "sslv3", "tls10", "tls11", "tls12", "tls13"]:
        assert config.get(ver) is True

    for feat in [
        "features",
        "compression",
        "dh_groups",
        "encrypt_then_mac",
        "ephemeral_key_reuse",
        "ext_master_secret",
        "fallback",
        "grease",
        "heartbeat",
        "ocsp_stapling",
        "renegotiation",
        "resumption",
    ]:
        assert config.get(feat) is True

    for vuln in [
        "vulnerabilities",
        "ccs_injection",
        "heartbleed",
        "padding_oracle",
        "robot",
    ]:
        assert config.get(vuln) is True

    assert config.get("oracle_accuracy") == "medium"

    for worker in [
        ScanCompression,
        ScanDhGroups,
        ScanEncryptThenMac,
        ScanEphemeralKeyReuse,
        ScanExtendedMasterSecret,
        ScanDowngrade,
        ScanGrease,
        ScanHeartbeat,
        ScanOcspStapling,
        ScanRenegotiation,
        ScanResumption,
        ScanCcsInjection,
        ScanHeartbleed,
        ScanPaddingOracle,
        ScanRobot,
        ScanStart,
        ScanCipherSuites,
        ScanSupportedGroups,
        ScanSigAlgs,
        ScanEnd,
    ]:
        assert worker in workers

    assert DumpProfileWorker not in workers
    assert ReadProfileWorker not in workers


def test_format_text(monkeypatch):
    monkeypatch.setattr(WorkManager, "run", work_manager_run)

    cmd = "tlsmate scan --port=1000 127.0.0.1 --format=text"
    sys.argv = cmd.split()
    command.main()
    config = TlsMate.instance.config
    workers = []
    _ = [workers.extend(pool) for pool in WorkManager._instance._prio_pool.values()]

    assert config.get("format") == "text"
    assert TextProfileWorker in workers
    assert DumpProfileWorker not in workers


def test_format_yaml(monkeypatch):
    monkeypatch.setattr(WorkManager, "run", work_manager_run)

    cmd = "tlsmate scan --port=1000 127.0.0.1 --format=yaml"
    sys.argv = cmd.split()
    command.main()
    config = TlsMate.instance.config
    workers = []
    _ = [workers.extend(pool) for pool in WorkManager._instance._prio_pool.values()]

    assert config.get("format") == "yaml"
    assert TextProfileWorker not in workers
    assert DumpProfileWorker in workers


def test_client_chain_no_key(capsys):
    cmd = "tlsmate scan --port=1000 127.0.0.1 --client-chain=xxx"
    sys.argv = cmd.split()
    with pytest.raises(SystemExit) as pytest_wrapped_e:
        command.main()
    captured = capsys.readouterr()
    assert "if --client-key is given, --client-chain must be given" in captured.err
    assert pytest_wrapped_e.value.code == 2


def test_client_chain_key_wrong(capsys):
    cmd = (
        "tlsmate scan --port=1000 127.0.0.1 " "--client-chain=xxx --client-key yyy zzz"
    )
    sys.argv = cmd.split()
    with pytest.raises(SystemExit) as pytest_wrapped_e:
        command.main()
    captured = capsys.readouterr()
    assert "number of arguments for --client-key and --client-chain" in captured.err
    assert pytest_wrapped_e.value.code == 2


def test_client_key_set(
    monkeypatch, client_rsa_key_filename, client_rsa_chain_filename
):
    monkeypatch.setattr(WorkManager, "run", work_manager_run)

    cmd = (
        f"tlsmate scan --port=1000 127.0.0.1 "
        f"--client-chain={client_rsa_chain_filename} "
        f"--client-key={client_rsa_key_filename}"
    )
    sys.argv = cmd.split()
    command.main()
    config = TlsMate.instance.config
    assert config.get("client_key") == [client_rsa_key_filename]
    assert config.get("client_chain") == [client_rsa_chain_filename]


def test_no_version(capsys):
    cmd = (
        "tlsmate scan --port=1000 127.0.0.1 "
        "--no-sslv2 --no-sslv3 --no-tls10 --no-tls11 --no-tls12 --no-tls13"
    )
    sys.argv = cmd.split()
    with pytest.raises(SystemExit) as pytest_wrapped_e:
        command.main()
    captured = capsys.readouterr()
    assert "at least one TLS version must be given" in captured.err
    assert pytest_wrapped_e.value.code == 2


def test_versions(monkeypatch):
    monkeypatch.setattr(WorkManager, "run", work_manager_run)

    cmd = "tlsmate scan --port=1000 127.0.0.1 --tls10 --tls12"
    sys.argv = cmd.split()
    command.main()
    config = TlsMate.instance.config
    workers = []
    _ = [workers.extend(pool) for pool in WorkManager._instance._prio_pool.values()]

    assert config.get("sslv2") is False
    assert config.get("sslv3") is False
    assert config.get("tls10") is True
    assert config.get("tls11") is False
    assert config.get("tls12") is True
    assert config.get("tls13") is False


def test_no_features(monkeypatch):
    monkeypatch.setattr(WorkManager, "run", work_manager_run)

    cmd = "tlsmate scan --port=1000 127.0.0.1 --no-features"
    sys.argv = cmd.split()
    command.main()
    config = TlsMate.instance.config
    workers = []
    _ = [workers.extend(pool) for pool in WorkManager._instance._prio_pool.values()]

    for feat in [
        "features",
        "compression",
        "dh_groups",
        "encrypt_then_mac",
        "ephemeral_key_reuse",
        "ext_master_secret",
        "fallback",
        "grease",
        "heartbeat",
        "ocsp_stapling",
        "renegotiation",
        "resumption",
    ]:
        assert config.get(feat) is False

    for worker in [
        ScanCompression,
        ScanDhGroups,
        ScanEncryptThenMac,
        ScanEphemeralKeyReuse,
        ScanExtendedMasterSecret,
        ScanDowngrade,
        ScanGrease,
        ScanHeartbeat,
        ScanOcspStapling,
        ScanRenegotiation,
        ScanResumption,
    ]:
        assert worker not in workers


def test_no_features2(monkeypatch):
    monkeypatch.setattr(WorkManager, "run", work_manager_run)

    cmd = (
        "tlsmate scan --port=1000 127.0.0.1 "
        "--no-features "
        "--no-compression "
        "--no-dh-groups "
        "--no-encrypt-then-mac "
        "--no-ephemeral-key-reuse "
        "--no-ext-master-secret "
        "--no-fallback "
        "--no-grease "
        "--no-heartbeat "
        "--no-ocsp-stapling "
        "--no-renegotiation "
        "--no-resumption "
    )
    sys.argv = cmd.split()
    command.main()
    config = TlsMate.instance.config
    workers = []
    _ = [workers.extend(pool) for pool in WorkManager._instance._prio_pool.values()]

    for feat in [
        "features",
        "compression",
        "dh_groups",
        "encrypt_then_mac",
        "ephemeral_key_reuse",
        "ext_master_secret",
        "fallback",
        "grease",
        "heartbeat",
        "ocsp_stapling",
        "renegotiation",
        "resumption",
    ]:
        assert config.get(feat) is False

    for worker in [
        ScanCompression,
        ScanDhGroups,
        ScanEncryptThenMac,
        ScanEphemeralKeyReuse,
        ScanExtendedMasterSecret,
        ScanDowngrade,
        ScanGrease,
        ScanHeartbeat,
        ScanOcspStapling,
        ScanRenegotiation,
        ScanResumption,
    ]:
        assert worker not in workers


def test_some_features(monkeypatch):
    monkeypatch.setattr(WorkManager, "run", work_manager_run)

    cmd = "tlsmate scan --port=1000 127.0.0.1 --no-features " "--compression --fallback"
    sys.argv = cmd.split()
    command.main()
    config = TlsMate.instance.config
    workers = []
    _ = [workers.extend(pool) for pool in WorkManager._instance._prio_pool.values()]

    for feat in [
        "features",
        "dh_groups",
        "encrypt_then_mac",
        "ephemeral_key_reuse",
        "ext_master_secret",
        "grease",
        "heartbeat",
        "ocsp_stapling",
        "renegotiation",
        "resumption",
    ]:
        assert config.get(feat) is False

    assert config.get("compression") is True
    assert config.get("fallback") is True

    for worker in [
        ScanDhGroups,
        ScanEncryptThenMac,
        ScanEphemeralKeyReuse,
        ScanExtendedMasterSecret,
        ScanGrease,
        ScanHeartbeat,
        ScanOcspStapling,
        ScanRenegotiation,
        ScanResumption,
    ]:
        assert worker not in workers

    assert ScanCompression in workers
    assert ScanDowngrade in workers


def test_not_some_features(monkeypatch):
    monkeypatch.setattr(WorkManager, "run", work_manager_run)

    cmd = "tlsmate scan --port=1000 127.0.0.1 " "--no-compression --no-fallback"
    sys.argv = cmd.split()
    command.main()
    config = TlsMate.instance.config
    workers = []
    _ = [workers.extend(pool) for pool in WorkManager._instance._prio_pool.values()]

    for feat in [
        "features",
        "dh_groups",
        "encrypt_then_mac",
        "ephemeral_key_reuse",
        "ext_master_secret",
        "grease",
        "heartbeat",
        "ocsp_stapling",
        "renegotiation",
        "resumption",
    ]:
        assert config.get(feat) is True

    assert config.get("compression") is False
    assert config.get("fallback") is False

    for worker in [
        ScanDhGroups,
        ScanEncryptThenMac,
        ScanEphemeralKeyReuse,
        ScanExtendedMasterSecret,
        ScanGrease,
        ScanHeartbeat,
        ScanOcspStapling,
        ScanRenegotiation,
        ScanResumption,
    ]:
        assert worker in workers

    assert ScanCompression not in workers
    assert ScanDowngrade not in workers


def test_no_vulnerabilities(monkeypatch):
    monkeypatch.setattr(WorkManager, "run", work_manager_run)

    cmd = "tlsmate scan --port=1000 127.0.0.1 --no-vulnerabilities"
    sys.argv = cmd.split()
    command.main()
    config = TlsMate.instance.config
    workers = []
    _ = [workers.extend(pool) for pool in WorkManager._instance._prio_pool.values()]

    for vuln in [
        "ccs_injection",
        "heartbleed",
        "padding_oracle",
        "robot",
    ]:
        assert config.get(vuln) is False

    for worker in [
        ScanCcsInjection,
        ScanHeartbleed,
        ScanPaddingOracle,
        ScanRobot,
    ]:
        assert worker not in workers


def test_no_vulnerabilities2(monkeypatch):
    monkeypatch.setattr(WorkManager, "run", work_manager_run)

    cmd = (
        "tlsmate scan --port=1000 127.0.0.1 "
        "--no-ccs-injection "
        "--no-heartbleed "
        "--no-padding-oracle "
        "--no-robot "
    )
    sys.argv = cmd.split()
    command.main()
    config = TlsMate.instance.config
    workers = []
    _ = [workers.extend(pool) for pool in WorkManager._instance._prio_pool.values()]

    for vuln in [
        "ccs_injection",
        "heartbleed",
        "padding_oracle",
        "robot",
    ]:
        assert config.get(vuln) is False

    for worker in [
        ScanCcsInjection,
        ScanHeartbleed,
        ScanPaddingOracle,
        ScanRobot,
    ]:
        assert worker not in workers


def test_some_vulnerabilities2(monkeypatch):
    monkeypatch.setattr(WorkManager, "run", work_manager_run)

    cmd = (
        "tlsmate scan --port=1000 127.0.0.1 --no-vulnerabilities "
        "--heartbleed "
        "--padding-oracle "
    )
    sys.argv = cmd.split()
    command.main()
    config = TlsMate.instance.config
    workers = []
    _ = [workers.extend(pool) for pool in WorkManager._instance._prio_pool.values()]

    for vuln in ["heartbleed", "padding_oracle"]:
        assert config.get(vuln) is True

    for vuln in ["ccs_injection", "robot"]:
        assert config.get(vuln) is False

    for worker in [ScanHeartbleed, ScanPaddingOracle]:
        assert worker in workers

    for worker in [ScanCcsInjection, ScanRobot]:
        assert worker not in workers


def test_not_some_vulnerabilities(monkeypatch):
    monkeypatch.setattr(WorkManager, "run", work_manager_run)

    cmd = (
        "tlsmate scan --port=1000 127.0.0.1 " "--no-heartbleed " "--no-padding-oracle "
    )
    sys.argv = cmd.split()
    command.main()
    config = TlsMate.instance.config
    workers = []
    _ = [workers.extend(pool) for pool in WorkManager._instance._prio_pool.values()]

    for vuln in ["heartbleed", "padding_oracle"]:
        assert config.get(vuln) is False

    for vuln in ["ccs_injection", "robot"]:
        assert config.get(vuln) is True

    for worker in [ScanHeartbleed, ScanPaddingOracle]:
        assert worker not in workers

    for worker in [ScanCcsInjection, ScanRobot]:
        assert worker in workers



# -*- coding: utf-8 -*-
"""Implements a class to be used for unit testing.
"""
import pathlib
from tlsmate.workers.supported_groups import ScanSupportedGroups
from tlsmate.tlssuite import TlsSuiteTester
from tlsmate.tlssuite import TlsLibrary


class TestCase(TlsSuiteTester):
    """Class used for tests with pytest.

    For more information refer to the documentation of the TcRecorder class.
    """

    sp_in_yaml = "profile_basic_openssl1_0_2"
    sp_out_yaml = "profile_supported_groups_openssl1_0_2"
    recorder_yaml = "recorder_supported_groups_openssl1_0_2"
    path = pathlib.Path(__file__)
    server_cmd = (
        "utils/start_openssl --version {library} --port {server_port} "
        "--cert1 server-rsa --cert2 server-ecdsa "
        "-- -www -cipher ALL"
    )
    library = TlsLibrary.openssl1_0_2

    server = "localhost"

    def check_ssl30(self, profile):
        assert profile["extension_supported"] == "FALSE"
        assert len(profile["groups"]) == 1
        assert profile["groups"][0]["name"] == "SECP256R1"

    def check_tls(self, profile):
        assert profile["extension_supported"] == "TRUE"
        assert profile["server_preference"] == "NA"
        assert profile["groups_advertised"] == "NA"
        assert len(profile["groups"]) == 1
        assert profile["groups"][0]["name"] == "SECP256R1"

    def check_profile(self, profile):
        self.check_ssl30(profile["versions"][1]["supported_groups"])
        self.check_tls(profile["versions"][2]["supported_groups"])
        self.check_tls(profile["versions"][3]["supported_groups"])
        self.check_tls(profile["versions"][4]["supported_groups"])

    def run(self, tlsmate, is_replaying):
        server_profile = tlsmate.server_profile
        ScanSupportedGroups(tlsmate).run()

        self.check_profile(server_profile.make_serializable())


if __name__ == "__main__":
    TestCase().entry(is_replaying=False)



# -*- coding: utf-8 -*-
"""Module scanning for resumption support
"""
# import basic stuff

# import own stuff
import tlsmate.msg as msg
import tlsmate.plugin as plg
import tlsmate.tls as tls

# import other stuff


class ScanResumption(plg.Worker):
    name = "resumption"
    descr = "scan for resumption support (session_id and ticket)"
    prio = 30

    def _resumption_tls12(self, prof_vals, session_ticket=False):
        if not prof_vals.cipher_suites:
            # no cipher suite, for which a hull handshake is supported.
            state = tls.ScanState.UNDETERMINED
        else:
            state = tls.ScanState.FALSE
            self.client.init_profile(profile_values=prof_vals)
            if session_ticket:
                self.client.profile.support_session_ticket = True
            else:
                self.client.profile.support_session_id = True
            with self.client.create_connection() as conn:
                conn.handshake()
            if conn.handshake_completed:
                if session_ticket:
                    resumption_possible = ...
                else:
                    resumption_possible = bool(len(conn.msg.server_hello.session_id))
                if resumption_possible:
                    self.client.profile.cipher_suites = [
                        conn.msg.server_hello.cipher_suite
                    ]
                    with self.client.create_connection() as conn2:
                        conn2.handshake()
                    if conn2.handshake_completed and conn2.abbreviated_hs:
                        state = tls.ScanState.TRUE
        return state

    def _run_tls12(self):
        versions = [tls.Version.TLS10, tls.Version.TLS11, tls.Version.TLS12]
        prof_vals = self.server_profile.get_profile_values(versions, full_hs=True)
        prof_features = self.server_profile.features

        session_id_support = tls.ScanState.NA
        session_ticket_support = tls.ScanState.NA
        if prof_vals.versions:
            session_id_support = self._resumption_tls12(prof_vals, session_ticket=False)
            session_ticket_support = self._resumption_tls12(
                prof_vals, session_ticket=True
            )

        prof_features.session_ticket = session_ticket_support
        prof_features.session_id = session_id_support
        if session_ticket_support is tls.ScanState.TRUE:
            prof_features.session_ticket_lifetime = (
                self.client.session.session_state_ticket.lifetime
            )

    def _run_tls13(self):
        resumption_psk = tls.ScanState.NA
        early_data = tls.ScanState.NA
        psk_lifetime = None
        prof_vals = self.server_profile.get_profile_values(
            [tls.Version.TLS13], full_hs=True
        )
        if prof_vals.versions:
            self.client.init_profile(profile_values=prof_vals)
            self.client.profile.support_psk = True
            self.client.profile.psk_key_exchange_modes = [
                tls.PskKeyExchangeMode.PSK_DHE_KE,
                tls.PskKeyExchangeMode.PSK_KE,
            ]
            ticket_msg = None
            with self.client.create_connection() as conn:
                conn.handshake()
                ticket_msg = conn.wait(msg.NewSessionTicket, optional=True, timeout=200)
                if ticket_msg is not None:
                    psk_lifetime = ticket_msg.lifetime
                else:
                    resumption_psk = tls.ScanState.FALSE

            if ticket_msg:
                resumption_psk = tls.ScanState.TRUE
                early_data = tls.ScanState.FALSE

                self.client.profile.early_data = b"This is EarlyData (0-RTT)"
                with self.client.create_connection() as conn:
                    conn.handshake()

                if conn.early_data_accepted:
                    early_data = tls.ScanState.TRUE

        self.server_profile.features.resumption_psk = resumption_psk
        self.server_profile.features.early_data = early_data
        if psk_lifetime is not None:
            self.server_profile.features.psk_lifetime = psk_lifetime

    def run(self):
        self.server_profile.allocate_features()
        self._run_tls12()
        self._run_tls13()



# -*- coding: utf-8 -*-
"""Implements a class to be used for unit testing.
"""
import pathlib
from tests.cipher_suite_tester import CipherSuiteTester
from tlsmate import tls
from tlsmate.tlssuite import TlsLibrary


class TestCase(CipherSuiteTester):
    """Class used for tests with pytest.

    For more information refer to the documentation of the CipherSuiteTester class.
    """

    path = pathlib.Path(__file__)
    name = "ClientAuth_ECDSA_SHA256"
    cipher_suite = tls.CipherSuite.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256
    server_cmd = (
        "utils/start_openssl --version {library} --port {server_port} "
        "--cert1 server-rsa --cert2 server-ecdsa "
        "-- -www -cipher ALL -verify 3 -client_sigalgs ECDSA+SHA256"
    )
    library = TlsLibrary.openssl1_1_1

    # Uncomment the line below if you do not want to use the default version and
    # adapt it to your needs.
    # version = tls.Version.TLS12


if __name__ == "__main__":
    TestCase().entry(is_replaying=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Script to extract the openssl s_server command line from a given pickle file
"""
# import basic stuff
import argparse

# import own stuff

# import other stuff
import pickle


def main():
    parser = argparse.ArgumentParser(description="openssl_command")
    parser.add_argument(
        "pickle_file", metavar="pickle-file", help="The pickle file to inspect"
    )
    args = parser.parse_args()

    with open(args.pickle_file, "rb") as fh:
        obj = pickle.load(fh)
    if hasattr(obj, "openssl_command"):
        if obj.openssl_command is not None:
            cmd = obj.openssl_command
            if isinstance(cmd, list):
                cmd = cmd[0]
            print("openssl_command: " + cmd)
            return
    print("No openssl command found.")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""Implements a class to be used for unit testing.
"""
import pathlib
from tlsmate.workers.eval_cipher_suites import ScanCipherSuites
from tlsmate.tlssuite import TlsSuiteTester
from tlsmate.tlssuite import TlsLibrary


tls12_cs = [
    "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA",
    "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA",
    "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256",
    "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384",
    "TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256",
    "TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384",
    "TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256",
]

tls13_cs = [
    "TLS_AES_128_GCM_SHA256",
    "TLS_AES_256_GCM_SHA384",
    "TLS_CHACHA20_POLY1305_SHA256",
]


class TestCase(TlsSuiteTester):
    """Class used for tests with pytest.

    For more information refer to the documentation of the TcRecorder class.
    """

    sp_out_yaml = "profile_basic_openssl3_0_0_ed25519_ed448"
    recorder_yaml = "recorder_eval_cipher_suites_openssl3_0_0_ed25519_ed448"
    path = pathlib.Path(__file__)
    server_cmd = (
        "utils/start_openssl --version {library} --port {server_port} "
        "--cert1 server-ed448 --cert2 server-ed25519 "
        "-- -www"
    )
    library = TlsLibrary.openssl3_0_0

    server = "localhost"

    def check_cert_chain(self, cert_chain):
        assert len(cert_chain) == 1
        assert cert_chain[0]["id"] == 1
        assert len(cert_chain[0]["cert_chain"]) == 2

    def check_versions(self, versions):
        assert len(versions) == 6
        assert versions[0]["version"]["name"] == "SSL20"
        assert versions[0]["support"] == "FALSE"

        assert versions[1]["version"]["name"] == "SSL30"
        assert versions[1]["support"] == "FALSE"

        assert versions[2]["version"]["name"] == "TLS10"
        assert versions[2]["support"] == "FALSE"

        assert versions[3]["version"]["name"] == "TLS11"
        assert versions[3]["support"] == "FALSE"

        assert versions[4]["version"]["name"] == "TLS12"
        assert versions[4]["support"] == "TRUE"
        assert versions[4]["ciphers"]["server_preference"] == "FALSE"

        assert versions[5]["version"]["name"] == "TLS13"
        assert versions[5]["support"] == "TRUE"
        assert versions[5]["ciphers"]["server_preference"] == "FALSE"
        for a, b in zip(tls12_cs, versions[4]["ciphers"]["cipher_suites"]):
            assert a == b["name"]
        for a, b in zip(tls13_cs, versions[5]["ciphers"]["cipher_suites"]):
            assert a == b["name"]

    def check_profile(self, profile):
        self.check_cert_chain(profile["cert_chains"])
        self.check_versions(profile["versions"])

    def run(self, tlsmate, is_replaying):
        for vers in ["sslv2", "sslv3", "tls10", "tls11", "tls12", "tls13"]:
            tlsmate.config.set(vers, True)
        server_profile = tlsmate.server_profile
        client = tlsmate.client
        client.init_profile()
        ScanCipherSuites(tlsmate).run()

        self.check_profile(server_profile.make_serializable())


if __name__ == "__main__":
    TestCase().entry(is_replaying=False)
